@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2008 Jason Woodward <woodwardj at jaos dot org>
+ * Copyright (C) 2003-2010 Jason Woodward <woodwardj at jaos dot org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,7 +73,7 @@ void slapt_pkg_action_install(const slapt_rc_config *global_config,
 
       if ( pkg == NULL ) {
         fprintf(stderr,gettext("No such package: %s\n"),arg);
-        continue;
+        exit(EXIT_FAILURE);
       }
 
     }
@@ -85,16 +85,17 @@ void slapt_pkg_action_install(const slapt_rc_config *global_config,
 
       if ( slapt_add_deps_to_trans(global_config,tran,avail_pkgs,
                                    installed_pkgs,pkg) == 0 ) {
-        slapt_pkg_info_t *conflicted_pkg = NULL;
 
-        /* if there is a conflict, we schedule the conflict for removal */
-        if ( (conflicted_pkg = slapt_is_conflicted(tran,avail_pkgs,
-                                                   installed_pkgs,
-                                                   pkg)) != NULL
-            && global_config->ignore_dep != SLAPT_TRUE
-            ) {
-          slapt_add_remove_to_transaction(tran,conflicted_pkg);
+        struct slapt_pkg_list *conflicts = slapt_is_conflicted(tran,avail_pkgs, installed_pkgs, pkg);
+
+        /* if there are conflicts, we schedule the conflicts for removal */
+        if (conflicts->pkg_count > 0 && global_config->ignore_dep != SLAPT_TRUE) {
+          unsigned int cindex = 0;
+          for (cindex = 0; cindex < conflicts->pkg_count;cindex++) {
+            slapt_add_remove_to_transaction(tran,conflicts->pkgs[cindex]);
+          }
         }
+        slapt_free_pkg_list(conflicts);
         slapt_add_install_to_transaction(tran,pkg);
 
       } else {
@@ -113,16 +114,17 @@ void slapt_pkg_action_install(const slapt_rc_config *global_config,
 
         if ( slapt_add_deps_to_trans(global_config,tran,avail_pkgs,
                                      installed_pkgs,pkg) == 0 ) {
-          slapt_pkg_info_t *conflicted_pkg = NULL;
 
-          if ( (conflicted_pkg = slapt_is_conflicted(tran,avail_pkgs,
-                                                     installed_pkgs,
-                                                     pkg)) != NULL
-            && global_config->ignore_dep != SLAPT_TRUE
-            ) {
-            slapt_add_remove_to_transaction(tran,conflicted_pkg);
+          struct slapt_pkg_list *conflicts = slapt_is_conflicted(tran,avail_pkgs,
+                                                     installed_pkgs, pkg);
+          if (conflicts->pkg_count > 0 && global_config->ignore_dep != SLAPT_TRUE) {
+            unsigned int cindex = 0;
+            for (cindex = 0;cindex < conflicts->pkg_count;cindex++) {
+              slapt_add_remove_to_transaction(tran,conflicts->pkgs[cindex]);
+            }
           }
           slapt_add_upgrade_to_transaction(tran,installed_pkg,pkg);
+          slapt_free_pkg_list(conflicts);
 
         } else {
           printf(gettext("Excluding %s, use --ignore-dep to override\n"),
@@ -429,6 +431,7 @@ void slapt_pkg_action_show(const char *pkg_name)
 
     printf(gettext("Package Name: %s\n"),pkg->name);
     printf(gettext("Package Mirror: %s\n"),pkg->mirror);
+    printf(gettext("Package Priority: %s\n"),slapt_priority_to_str(pkg->priority));
     printf(gettext("Package Location: %s\n"),pkg->location);
     printf(gettext("Package Version: %s\n"),pkg->version);
     printf(gettext("Package Size: %d K\n"),pkg->size_c);
@@ -484,8 +487,9 @@ void slapt_pkg_action_upgrade_all(const slapt_rc_config *global_config)
   tran = slapt_init_transaction();
 
   if ( global_config->dist_upgrade == SLAPT_TRUE ) {
-    char *essential[] = {"glibc-solibs","sed","pkgtools",NULL};
+    char *essential[] = {"pkgtools","xz","glibc-solibs","sed","tar","gzip",NULL};
     int epi = 0;
+    slapt_pkg_info_t *newest_slaptget = NULL;
     struct slapt_pkg_list *matches =
       slapt_search_pkg_list(avail_pkgs,SLAPT_SLACK_BASE_SET_REGEX);
 
@@ -539,15 +543,14 @@ void slapt_pkg_action_upgrade_all(const slapt_rc_config *global_config)
         if ( slapt_is_excluded(global_config,slapt_upgrade_pkg) == 1 ) {
           slapt_add_exclude_to_transaction(tran,slapt_upgrade_pkg);
         } else {
+          struct slapt_pkg_list *conflicts = slapt_is_conflicted(tran,avail_pkgs,installed_pkgs,
+                                     slapt_upgrade_pkg);
 
           /* add install if all deps are good and it doesn't have conflicts */
           if (
             (slapt_add_deps_to_trans(global_config,tran,avail_pkgs,
                                      installed_pkgs,slapt_upgrade_pkg) == 0)
-            && ( slapt_is_conflicted(tran,avail_pkgs,installed_pkgs,
-                                     slapt_upgrade_pkg) == NULL
-            && global_config->ignore_dep != SLAPT_TRUE
-            )
+            && ( conflicts->pkg_count == 0 && global_config->ignore_dep != SLAPT_TRUE)
           ) {
             slapt_add_install_to_transaction(tran,slapt_upgrade_pkg);
           } else {
@@ -557,6 +560,8 @@ void slapt_pkg_action_upgrade_all(const slapt_rc_config *global_config)
             slapt_add_exclude_to_transaction(tran,slapt_upgrade_pkg);
           }
 
+          slapt_free_pkg_list(conflicts);
+
         }
       /* even if it's installed, check to see that the packages are different */
       /* simply running a version comparison won't do it since sometimes the */
@@ -570,13 +575,14 @@ void slapt_pkg_action_upgrade_all(const slapt_rc_config *global_config)
             || slapt_is_excluded(global_config,slapt_upgrade_pkg) == 1 ) {
           slapt_add_exclude_to_transaction(tran,slapt_upgrade_pkg);
         } else {
+          struct slapt_pkg_list *conflicts = slapt_is_conflicted(tran,avail_pkgs,installed_pkgs,
+                                     slapt_upgrade_pkg);
+
           /* if all deps are added and there is no conflicts, add on */
           if (
             (slapt_add_deps_to_trans(global_config,tran,avail_pkgs,
                                      installed_pkgs,slapt_upgrade_pkg) == 0)
-            && ( slapt_is_conflicted(tran,avail_pkgs,installed_pkgs,
-                                     slapt_upgrade_pkg) == NULL
-            && global_config->ignore_dep != SLAPT_TRUE)
+            && ( conflicts->pkg_count == 0 && global_config->ignore_dep != SLAPT_TRUE)
           ) {
             slapt_add_upgrade_to_transaction(tran,installed_pkg,
                                              slapt_upgrade_pkg);
@@ -586,6 +592,8 @@ void slapt_pkg_action_upgrade_all(const slapt_rc_config *global_config)
                    slapt_upgrade_pkg->name);
             slapt_add_exclude_to_transaction(tran,slapt_upgrade_pkg);
           }
+
+          slapt_free_pkg_list(conflicts);
         }
 
       }
@@ -615,10 +623,13 @@ void slapt_pkg_action_upgrade_all(const slapt_rc_config *global_config)
     }/* end if remove_obsolete */
 
     /* insurance so that all of slapt-get's requirements are also installed */
-    slapt_add_deps_to_trans(
-      global_config,tran,avail_pkgs,installed_pkgs,
-      slapt_get_newest_pkg(avail_pkgs,"slapt-get")
-    );
+    newest_slaptget =  slapt_get_newest_pkg(avail_pkgs,"slapt-get");
+    if (newest_slaptget != NULL) {
+      slapt_pkg_info_t *installed_slaptget = slapt_get_newest_pkg(installed_pkgs, "slapt-get");
+      slapt_add_deps_to_trans(global_config, tran, avail_pkgs, installed_pkgs, newest_slaptget);
+      if (installed_slaptget != NULL)
+        slapt_add_upgrade_to_transaction(tran,installed_slaptget, newest_slaptget);
+    }
     
   }
 
@@ -668,13 +679,15 @@ void slapt_pkg_action_upgrade_all(const slapt_rc_config *global_config)
         ) {
           slapt_add_exclude_to_transaction(tran,update_pkg);
         } else {
+          struct slapt_pkg_list *conflicts = slapt_is_conflicted(tran,
+                                    avail_pkgs,installed_pkgs, update_pkg);
+
           /* if all deps are added and there is no conflicts, add on */
           if (
             (slapt_add_deps_to_trans(global_config,tran,avail_pkgs,
                                      installed_pkgs,update_pkg) == 0)
             && ( global_config->ignore_dep == SLAPT_TRUE
-            || ( slapt_is_conflicted(tran,avail_pkgs,installed_pkgs,
-                                     update_pkg) == NULL ))
+            || ( conflicts->pkg_count == 0 ) )
           ) {
             slapt_add_upgrade_to_transaction(tran,installed_pkgs->pkgs[i],
                                              update_pkg);
@@ -684,6 +697,8 @@ void slapt_pkg_action_upgrade_all(const slapt_rc_config *global_config)
                    update_pkg->name);
             slapt_add_exclude_to_transaction(tran,update_pkg);
           }
+
+          slapt_free_pkg_list(conflicts);
         }
 
       }
@@ -781,10 +796,9 @@ void slapt_pkg_action_add_keys(const slapt_rc_config *global_config)
   for(s = 0; s < global_config->sources->count; s++)
   {
     FILE *gpg_key = NULL;
-    printf(gettext("Retrieving GPG key [%s]..."), global_config->sources->url[s]);
-    gpg_key = slapt_get_pkg_source_gpg_key (global_config,
-                                            global_config->sources->url[s],
-                                            &compressed);
+    const char *source_url = global_config->sources->src[s]->url;
+    printf(gettext("Retrieving GPG key [%s]..."), source_url);
+    gpg_key = slapt_get_pkg_source_gpg_key (global_config, source_url, &compressed);
     if (gpg_key != NULL)
     {
       slapt_code_t r = slapt_add_pkg_source_gpg_key(gpg_key);
@@ -803,3 +817,48 @@ void slapt_pkg_action_add_keys(const slapt_rc_config *global_config)
 
 }
 #endif
+
+void slapt_pkg_action_filelist( const char *pkg_name )
+{
+  slapt_regex_t *pkg_regex = NULL;
+  struct slapt_pkg_list *installed_pkgs = NULL;
+  slapt_pkg_info_t *pkg = NULL;
+  char *filelist = NULL;
+
+  installed_pkgs = slapt_get_installed_pkgs();
+  if ( installed_pkgs == NULL )
+    exit(EXIT_FAILURE);
+
+  if ((pkg_regex = slapt_init_regex(SLAPT_PKG_LOG_PATTERN)) == NULL)
+    exit(EXIT_FAILURE);
+
+  /* Use regex to see if they specified a particular version */
+  slapt_execute_regex(pkg_regex,pkg_name);
+
+  /* If so, parse it out and try to get that version only */
+  if ( pkg_regex->reg_return == 0 ) {
+    char *p_name,*p_version;
+
+    p_name    = slapt_regex_extract_match(pkg_regex, pkg_name, 1);
+    p_version = slapt_regex_extract_match(pkg_regex, pkg_name, 2);
+
+    pkg = slapt_get_exact_pkg(installed_pkgs,p_name,p_version);
+
+    if ( pkg == NULL )
+      exit(EXIT_FAILURE);
+
+    free(p_name);
+    free(p_version);
+
+  } else {
+    pkg = slapt_get_newest_pkg(installed_pkgs,pkg_name);
+    if ( pkg == NULL )
+      exit(EXIT_FAILURE);
+  }
+
+  filelist = slapt_get_pkg_filelist(pkg);
+
+  printf("%s",filelist);
+
+  free(filelist);
+}
